# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 16.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V32
2677
2678
3772
3774
4155
4159
4190
4193
4226
4227
6741
6743
43188
43189
43453
43456
43571
43575
71453
71456
72000
72001
72002
72003
72251
72255
73031
73032
90410
90413
90414
90415
END
