# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 16.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V32
3302
3312
7410
7411
7412
7413
43056
43062
43249
43250
70528
70538
70539
70540
70542
70543
70544
70582
70583
70593
70594
70595
70597
70598
70599
70603
70604
70614
70615
70617
70625
70627
END
